package cn.com.ctop.kuaishou.modules.graphql.service.impl;

import cn.com.ctop.common.utils.HttpUtils;
import cn.com.ctop.kuaishou.modules.graphql.entity.*;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouUserService;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouVideoService;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouWebInterfaceService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.apache.http.util.EntityUtils;
import org.openqa.selenium.Cookie;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class KuaishouWebInterfaceServiceImpl implements IKuaishouWebInterfaceService {
    @Autowired
    private IKuaishouUserService kuaishouUserService;
    @Autowired
    private IKuaishouVideoService kuaishouVideoService;
    public static String GRAPHQL_URL = "https://live.kuaishou.com/graphql";
    public void getSearchUserList(String keyword,int page,String ttf){
        String jsonStr = "{\"operationName\":\"SearchDetailQuery\",\"variables\":{\"key\":\""+keyword+"\",\"type\":\"author\",\"page\":"+String.valueOf(page)+",\"lssid\":null,\"ussid\":\"MV8wXzE1NjA4NTgzMjMzODdf5ri45oiPXzk3MTE\"},\"query\":\"query SearchDetailQuery($key: String, $type: String, $page: Int, $lssid: String, $ussid: String) {  searchDetail(key: $key, type: $type, page: $page, lssid: $lssid, ussid: $ussid) {    ... on SearchCategoryList {      type      list {        id        categoryId        title        src        roomNumber        __typename      }      __typename    }    ... on SearchUserList {      type      ussid      list {        id        name        living        profile        sex        description        countsInfo {          fan          follow          photo          __typename        }        __typename      }      __typename    }    ... on SearchLivestreamList {      type      lssid      list {        user {          id          profile          name          __typename        }        watchingCount        src        title        gameId        gameName        categoryId        liveStreamId        playUrls {          quality          url          __typename        }        quality        gameInfo {          category          name          pubgSurvival          type          kingHero          __typename        }        redPack        liveGuess        expTag        __typename      }      __typename    }    __typename  }}\"}";
        try {
            if(page < 1){
                page = 1;
            }
            String result = HttpUtils.callingGraph(GRAPHQL_URL,jsonStr);
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            for(String key : map.keySet()){
                result = result.replace(key,map.get(key));
            }
            System.out.println(result);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            KuaishouWebResult<KuaishouSearchResult> data = mapper.readValue(result,new TypeReference<KuaishouWebResult<KuaishouSearchResult>>(){});
            List<KuaishouAuthors> list = data.getData().getSearchDetail().getList();
            if(list != null && list.size() > 0){
                System.out.println("list size="+list.size());
                Thread.sleep((int)Math.random()*5000+5000);
                getSearchUserList(keyword,++page,ttf);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void getVedioList(String principalId,String pcursor,String ttf,int times){
        String jsonStr = "{" +
                "\"operationName\":\"publicFeedsQuery\"," +
                "\"variables\":{\"principalId\":\""+principalId+"\",\"pcursor\":\""+pcursor+"\",\"count\":24}," +
                "\"query\":\"query publicFeedsQuery($principalId: String, $pcursor: String, $count: Int) " +
                "{" +
                "publicFeeds(principalId: $principalId, pcursor: $pcursor, count: $count) " +
                "{    " +
                "pcursor    " +
                "live " +
                "{      " +
                "user " +
                "{        " +
                "id        " +
                "kwaiId        " +
                "eid        " +
                "profile        " +
                "name        " +
                "living        " +
                "}      " +
                "watchingCount      " +
                "src      " +
                "title      " +
                "gameId      " +
                "gameName      " +
                "categoryId      " +
                "liveStreamId      " +
                "playUrls " +
                "{        " +
                "quality        " +
                "url        " +
                "}      " +
                "followed      " +
                "type      " +
                "living      " +
                "redPack      " +
                "liveGuess      " +
                "anchorPointed      " +
                "latestViewed      " +
                "expTag      " +
                "}    " +
                "list " +
                "{      " +
                "photoId      " +
                "caption      " +
                "thumbnailUrl      " +
                "poster      " +
                "viewCount      " +
                "likeCount      " +
                "commentCount      " +
                "timestamp      " +
                "workType      " +
                "type      " +
                "useVideoPlayer      " +
                "imgUrls      " +
                "imgSizes      " +
                "magicFace      " +
                "musicName      " +
                "location      " +
                "liked      " +
                "onlyFollowerCanComment      " +
                "relativeHeight      " +
                "width      " +
                "height      " +
                "user " +
                "{        " +
                "id        " +
                "eid        " +
                "name        " +
                "profile        " +
                "}      " +
                "expTag      " +
                "}    " +
                "}" +
                "}\"" +
                "}";
        try {
            if (times > 5){
                return;
            }
            String result = HttpUtils.callingGraph(GRAPHQL_URL,jsonStr);
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            for(String key : map.keySet()){
                result = result.replace(key,map.get(key));
            }
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            KuaishouWebResult<KuaishouWebData> data = mapper.readValue(result,new TypeReference<KuaishouWebResult<KuaishouWebData>>(){});
            KuaishouPublicFeeds feeds = data.getData().getPublicFeeds();
            if(feeds.getList() != null && feeds.getList().size() > 0){
                List<KuaishouVideo> videoList = feeds.getList();
                for(KuaishouVideo video : videoList){
                    video.setUserId(video.getUser().getId());
                    video.setTimes(new Date(video.getTimestamp()));
                    video.setTtfName(ttf);
                    kuaishouUserService.saveOrUpdate(video.getUser());
                    kuaishouVideoService.saveOrUpdate(video);
                }
                System.out.println("list size="+feeds.getList().size());
            }
            if (feeds.getPcursor() == null || !feeds.getPcursor().equals("no_more")){
                Thread.sleep((int)Math.random()*5000+10000);
                getVedioList(principalId,feeds.getPcursor(),ttf,++times);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void getSecurity(String url){
        String detectionPoint = "{\"a\":false,\"b\":true,\"c\":true,\"d\":true,\"e\":false,\"f\":false,\"g\":false,\"h\":true,\"i\":true,\"j\":false,\"k\":false,\"l\":true,\"m\":true,\"n\":true,\"o\":false,\"p\":false,\"q\":true,\"r\":false,\"s\":false,\"t\":false,\"u\":false,\"v\":false,\"w\":false,\"x\":false,\"y\":false,\"z\":true,\"aa\":false,\"bb\":false,\"cc\":false,\"dd\":false,\"ee\":false,\"ff\":false,\"gg\":false,\"hh\":false,\"ii\":false,\"jj\":false,\"kk\":false,\"ll\":false,\"mm\":false,\"nn\":false,\"oo\":false,\"pp\":1}";
        String collectInfo="{\"a\":\"live.kuaishou.com\",\"b\":\""+url+"\",\"c\":\"null的主页-快手直播\",\"d\":\"\",\"e\":\""+HttpUtils.USER_AGENT+"\",\"f\":1920,\"g\":1080,\"h\":24,\"i\":\"zh-CN\"}";
        try {
            String securityUrl = "https://live.kuaishou.com/rest/wd/live/web/security?detectionPoint="+URLEncoder.encode(detectionPoint,"UTF-8")+"&collectInfo="+URLEncoder.encode(collectInfo,"UTF-8");
            String html2 = HttpUtils.httpGetRequest(securityUrl);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    public void getVideoList(String uid){
//        System.getProperties().setProperty("webdriver.chrome.driver", "D:/chromedriver.exe");
        System.getProperties().setProperty("webdriver.chrome.driver", "/usr/bin/chromedriver");
        try {
            ChromeOptions chromeOptions = new ChromeOptions();
            WebDriver webDriver = new ChromeDriver(chromeOptions);
            chromeOptions.addArguments("--headless");
            chromeOptions.addArguments("--no-sandbox");
            chromeOptions.addArguments("--disable-gpu");
            chromeOptions.addArguments("--window-size=1290,1080");
            chromeOptions.addArguments("--user-agent="+HttpUtils.USER_AGENT);
            chromeOptions.setAcceptInsecureCerts(true);
            webDriver.get("https://live.kuaishou.com/profile/"+uid);
            Thread.sleep(2000);
            webDriver.navigate().refresh();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (Cookie cookie : webDriver.manage().getCookies()){
                BasicClientCookie ck = new BasicClientCookie(cookie.getName(),cookie.getValue());
                ck.setDomain("live.kuaishou.com");
//                ck.setDomain(cookie.getDomain());
                ck.setExpiryDate(cookie.getExpiry());
                ck.setPath(cookie.getPath());
                HttpUtils.cookieStore.addCookie(ck);
           }
            String html = webDriver.getPageSource();
            Pattern p=Pattern.compile("fontscn\\w+\\.ttf");
            Matcher m=p.matcher(html);

            String ttf = "";
            while(m.find()) {
                ttf = m.group();
                System.out.println("ttf:"+ttf);
            }
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            if(map == null){
                HttpUtils.kuaishouTtf(ttf);
            }
            getVedioList(uid,"",ttf,1);
            webDriver.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setCookies(){
        HttpUtils.cookieStore.clear();
        KuaishouWebInterfaceServiceImpl i= new KuaishouWebInterfaceServiceImpl();
        BasicClientCookie cookie = new BasicClientCookie("clientid", "3");
        cookie.setDomain("live.kuaishou.com");   //设置范围
        cookie.setPath("/");
        cookie.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie);

        BasicClientCookie cookie1 = new BasicClientCookie("did", "web_1615d3ad757d4fb8558ed4d7dbecf5fc");
        cookie1.setDomain("live.kuaishou.com");   //设置范围web_a5735c4f9f02ad9ee48677499927341c
        cookie1.setPath("/");
        cookie1.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie1);

        BasicClientCookie cookie2 = new BasicClientCookie("client_key", "65890b29");
        cookie2.setDomain("live.kuaishou.com");   //设置范围
        cookie2.setPath("/");
        cookie2.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie2);

        BasicClientCookie cookie3 = new BasicClientCookie("kuaishou.live.bfb1s", "3e261140b0cf7444a0ba411c6f227d88");
        cookie3.setDomain("live.kuaishou.com");   //设置范围
        cookie3.setPath("/");
        cookie3.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie3);
    }

    public String checkKuaishouUser(String videoUrl){
        String html = HttpUtils.httpGetRequest(videoUrl);
        HttpUtils.cookieStore.clear();
        Pattern p=Pattern.compile("/profile/(\\w+)\"");
        Matcher m=p.matcher(html);
        String uid = "";

        while(m.find()) {
            uid = m.group().replace("/profile/","").replace("\"","");
            if (uid != null && !uid.equals("")&& !uid.equals("null")){
                return uid;
            }
        }
        return null;
    }
    public static void main(String[] args){
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpGet httpGet = new HttpGet("http://api.gifshow.com/rest/n/feed/hot?app=0&kpf=ANDROID_PHONE&ver=6.1&c=XIAOMI&mod=Xiaomi%28MIX%202%29&appver=6.1.1.8096&ftt=K-T-T&isp=CUCC&kpn=KUAISHOU&lon=121.307828&language=zh-cn&sys=ANDROID_8.0.0&max_memory=256&ud=1211439612&country_code=cn&oc=XIAOMI&hotfix_ver=&did_gt=1547968116027&iuid=&extId=c37f259c4d56246ff655c4627550fa26&net=WIFI&did=ANDROID_75eb73714a8769c6&lat=31.219055&type=7&page=1&coldStart=false&count=20&pv=false&id=14363&refreshTimes=2&pcursor=&source=1&needInterestTag=false&browseType=1&os=android&__NStokensig=da69a950da90cedfef80ebce0ca028d17df48314869431ad872333a97994e3e5&token=ecd7394b3383474a9f41098ce22b5b01-1211439612&sig=4070cf6a1c277303ccb700456848c447&client_key=3c2cd3f3");
//            HttpGet httpGet = new HttpGet("http://api.gifshow.com/rest/n/feed/hot?app=0&kpf=ANDROID_PHONE&ver=6.5&c=HUAWEI&mod=HUAWEI%28ALP-AL00%29&appver=6.5.4.9545&ftt=&isp=CUCC&kpn=KUAISHOU&lon=116.489615&language=zh-cn&sys=ANDROID_9&max_memory=384&ud=1153842485&country_code=cn&pm_tag=&oc=HUAWEI&hotfix_ver=&did_gt=1559122744187&iuid=&extId=fcee8ea75e26da2bd350e97b16ab1ec8&net=WIFI&did=ANDROID_99fc74cb7188a11f&lat=39.958759&type=7&page=29&coldStart=false&count=20&pv=false&id=188&refreshTimes=32&pcursor=1&source=1&needInterestTag=false&browseType=1&seid=661bd5f2-eb7a-41d3-9d42-8b527e273336&volume=0.0&os=android&__NStokensig=2f84d7e5cced2c4570b6ec3b9098a2b3b037e2747a54c5bf994c25c6b98ae068&token=5ec7b1848e6c4e1e9ad4a2004e904357-1153842485&sig=61f6bc0b74a3a39e16bfb732e0c39cc3&client_key=3c2cd3f3");
            CloseableHttpResponse response = httpClient.execute(httpGet);

            String html=null;
            if(response.getStatusLine().getStatusCode()==200){
                html = EntityUtils.toString(response.getEntity(), "UTF-8");
            }
            System.out.println(html);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
